@@ -1007,16 +1007,12 @@ try_update_scene_state( struct lp_setup_context *setup )
                                          &setup->draw_regions[i]);
          }
       }
-      /* If the framebuffer is large we have to think about fixed-point
-       * integer overflow.  For 2K by 2K images, coordinates need 15 bits
-       * (2^11 + 4 subpixel bits).  The product of two such numbers would
-       * use 30 bits.  Any larger and we could overflow a 32-bit int.
-       *
-       * To cope with this problem we check if triangles are large and
-       * subdivide them if needed.
+      /*
+       * Subdivide triangles if the framebuffer is larger than the
+       * MAX_FIXED_LENGTH.
        */
-      setup->subdivide_large_triangles = (setup->fb.width > 2048 &&
-                                          setup->fb.height > 2048);
+      setup->subdivide_large_triangles = (setup->fb.width > MAX_FIXED_LENGTH ||
+                                          setup->fb.height > MAX_FIXED_LENGTH);
    }
                                       
    setup->dirty = 0;
@@ -1053,6 +1049,7 @@ lp_setup_update_state( struct lp_setup_context *setup,
       setup->psize = lp->psize_slot;
       setup->viewport_index_slot = lp->viewport_index_slot;
       setup->layer_slot = lp->layer_slot;
+      setup->face_slot = lp->face_slot;
 
       assert(lp->dirty == 0);
 
